import { echo } from "@atums/echo";
import { cassandraConfig as config } from "#environment/database";
import { noFileLog } from "#index";

import {
	Client,
	type DseClientOptions,
	type QueryOptions,
	auth,
} from "cassandra-driver";
import { environment } from "#environment/config";
import type { ConnectionOptions } from "#types/config";

class CassandraService {
	private static instance: Client | null = null;
	private static isConnecting = false;
	private static connectionPromise: Promise<void> | null = null;

	private constructor() {}

	public static getClient(): Client {
		if (!CassandraService.instance) {
			throw new Error(
				"Cassandra client is not initialized. Call connect() first.",
			);
		}
		return CassandraService.instance;
	}

	public static isConnected(): boolean {
		return (
			CassandraService.instance !== null &&
			CassandraService.instance.getState().getConnectedHosts().length > 0
		);
	}

	private static buildClientOptions(
		options: ConnectionOptions,
	): DseClientOptions {
		const { withKeyspace = true, timeout = 30000 } = options;

		const authProvider = config.authEnabled
			? new auth.PlainTextAuthProvider(config.username, config.password)
			: undefined;

		const clientOptions: DseClientOptions = {
			contactPoints: config.contactPoints,
			localDataCenter: config.datacenter,
			protocolOptions: {
				port: config.port,
			},
			socketOptions: {
				connectTimeout: timeout,
				readTimeout: timeout,
			},
		};

		if (authProvider) {
			clientOptions.authProvider = authProvider;
		}

		if (withKeyspace && config.keyspace) {
			clientOptions.keyspace = config.keyspace;
		}

		return clientOptions;
	}

	public static async connect(options: ConnectionOptions = {}): Promise<void> {
		if (
			CassandraService.instance &&
			CassandraService.instance.getState().getConnectedHosts().length > 0
		) {
			return;
		}

		if (CassandraService.isConnecting && CassandraService.connectionPromise) {
			return CassandraService.connectionPromise;
		}

		CassandraService.isConnecting = true;

		try {
			CassandraService.connectionPromise =
				CassandraService.performConnection(options);
			await CassandraService.connectionPromise;
		} finally {
			CassandraService.isConnecting = false;
			CassandraService.connectionPromise = null;
		}
	}

	private static async performConnection(
		options: ConnectionOptions,
	): Promise<void> {
		const clientOptions = CassandraService.buildClientOptions(options);

		if (options.logging !== false) {
			noFileLog.info({
				message: "Connecting to Cassandra...",
				contactPoints: config.contactPoints,
				datacenter: config.datacenter,
				keyspace: clientOptions.keyspace || "none",
				authEnabled: config.authEnabled,
			});
		}

		const client = new Client(clientOptions);

		try {
			await client.connect();

			const hosts = client.getState().getConnectedHosts();
			const hostCount = hosts.length;

			if (options.logging !== false) {
				noFileLog.info(
					`Connected to Cassandra successfully. Active hosts: ${hostCount}`,
				);
			}

			CassandraService.instance = client;

			if (options.logging !== false) {
				client.on(
					"log",
					(level: string, className: string, message: string) => {
						if (level === "error") {
							echo.error(`Cassandra ${className}: ${message}`);
						} else if (level === "warning") {
							echo.warn(`Cassandra ${className}: ${message}`);
						}
					},
				);
			}
		} catch (error) {
			echo.error({ message: "Failed to connect to Cassandra:", error });
			await client.shutdown().catch(() => {});
			throw error;
		}
	}

	public static async createKeyspaceIfNotExists(): Promise<void> {
		if (!config.keyspace) {
			throw new Error("No keyspace configured");
		}

		const client = CassandraService.getClient();

		const query = `
			CREATE KEYSPACE IF NOT EXISTS ${config.keyspace}
			WITH REPLICATION = {
				'class': 'SimpleStrategy',
				'replication_factor': 1
			}
		`;

		try {
			await client.execute(query);
			noFileLog.debug(`Keyspace '${config.keyspace}' ensured to exist`);
		} catch (error) {
			echo.error({
				message: `Failed to create keyspace '${config.keyspace}':`,
				error,
			});
			throw error;
		}
	}

	public static async execute(
		query: string,
		params?: unknown[],
		options?: QueryOptions,
	): Promise<unknown> {
		const client = CassandraService.getClient();

		try {
			const result = await client.execute(query, params, options);
			return result;
		} catch (error) {
			echo.error({
				message: "Cassandra query failed:",
				query: query.substring(0, 100) + (query.length > 100 ? "..." : ""),
				error,
			});
			throw error;
		}
	}

	public static async shutdown(disableLogging = false): Promise<void> {
		if (CassandraService.instance) {
			try {
				await CassandraService.instance.shutdown();
				if (!disableLogging) {
					noFileLog.info("Cassandra client shut down gracefully");
				}
			} catch (error) {
				echo.error({ message: "Error during Cassandra shutdown:", error });
			} finally {
				CassandraService.instance = null;
			}
		}
	}

	public static getHealthStatus(): {
		connected: boolean;
		hosts: number;
	} {
		if (!CassandraService.instance) {
			return { connected: false, hosts: 0 };
		}

		const hosts = CassandraService.instance.getState().getConnectedHosts();
		return {
			connected: hosts.length > 0,
			hosts: hosts.length,
		};
	}

	public static async dropEverything(): Promise<void> {
		if (!config.keyspace) {
			throw new Error("No keyspace configured");
		}

		if (!environment.development)
			throw new Error(
				"Drop operation is only allowed in development environment",
			);

		const client = CassandraService.getClient();

		try {
			const tablesQuery = `
			SELECT table_name FROM system_schema.tables
			WHERE keyspace_name = ?
		`;
			const tablesResult = await client.execute(tablesQuery, [config.keyspace]);

			const tableNames = tablesResult.rows.map((row) => {
				const tableRow = row as unknown as { table_name: string };
				return tableRow.table_name;
			});

			if (tableNames.length > 0) {
				noFileLog.warn(
					`About to drop keyspace '${config.keyspace}' containing tables: ${tableNames.join(", ")}`,
				);
			} else {
				noFileLog.info(
					`Keyspace '${config.keyspace}' is empty or doesn't exist`,
				);
			}

			const dropQuery = `DROP KEYSPACE IF EXISTS ${config.keyspace}`;
			await client.execute(dropQuery);

			noFileLog.info(`Keyspace '${config.keyspace}' dropped successfully`);
		} catch (error) {
			echo.error({
				message: `Failed to drop keyspace '${config.keyspace}':`,
				error,
			});
			throw error;
		}

		if (CassandraService.instance) {
			try {
				await CassandraService.shutdown(true);
			} catch (shutdownError) {
				noFileLog.warn({
					message: "Error during shutdown after drop:",
					error: shutdownError,
				});
			}
		}

		CassandraService.instance = null;
		CassandraService.isConnecting = false;
		CassandraService.connectionPromise = null;

		noFileLog.info("Cassandra client state reset after dropping keyspace");
	}

	public static async resetDatabase(): Promise<void> {
		if (!environment.development)
			throw new Error(
				"Reset operation is only allowed in development environment",
			);

		noFileLog.info("Starting database reset...");

		await CassandraService.dropEverything();

		await CassandraService.connect({ withKeyspace: false, logging: true });
		await CassandraService.createKeyspaceIfNotExists();
		await CassandraService.shutdown(true);

		noFileLog.info(
			"Database reset complete. Restart your application to run migrations.",
		);
	}
}

export { CassandraService as cassandra };
