import { readdir } from "node:fs/promises";
import { echo } from "@atums/echo";
import nodemailer from "nodemailer";
import { templatesPath } from "#environment/constants";
import { mailerConfig } from "#environment/mailer";
import { noFileLog } from "#index";

import type {
	EmailResult,
	SendMailOptions,
	TemplateVariables,
} from "#types/lib";

class MailerService {
	private transporter: nodemailer.Transporter;

	constructor() {
		this.transporter = nodemailer.createTransport({
			host: mailerConfig.address,
			port: mailerConfig.port,
			secure: mailerConfig.secure,
			auth: {
				user: mailerConfig.username,
				pass: mailerConfig.password,
			},
			pool: mailerConfig.pool,
			maxConnections: mailerConfig.maxConnections,
			maxMessages: mailerConfig.maxMessages,
			connectionTimeout: mailerConfig.connectionTimeout,
			socketTimeout: mailerConfig.socketTimeout,
			requireTLS: !mailerConfig.secure,
			tls: {
				rejectUnauthorized: true,
			},
		} as nodemailer.TransportOptions);
	}

	async verifyConnection(): Promise<boolean> {
		try {
			await this.transporter.verify();
			noFileLog.info("SMTP connection verified successfully");
			return true;
		} catch (error) {
			echo.error({ message: "SMTP connection verification failed:", error });
			return false;
		}
	}

	async close(): Promise<void> {
		this.transporter.close();
	}

	private async listTemplates(): Promise<string[]> {
		try {
			const files = await readdir(templatesPath, { recursive: true });
			return files
				.filter((file) => typeof file === "string" && file.endsWith(".html"))
				.map((file) => (file as string).replace(/\.html$/, ""));
		} catch (error) {
			echo.error({ message: "Failed to list templates:", error });
			return [];
		}
	}

	private async renderTemplate(
		templateName: string,
		variables: TemplateVariables,
	): Promise<string> {
		const templates = await this.listTemplates();
		if (!templates.includes(templateName)) {
			throw new Error(`Template "${templateName}" not found`);
		}

		const templatePath = `${templatesPath}/${templateName}.html`;
		let templateContent: string;

		try {
			const file = Bun.file(templatePath);
			templateContent = await file.text();
		} catch (error) {
			echo.error({
				message: `Failed to load template "${templateName}":`,
				error,
			});
			throw error;
		}

		const rewriter = new HTMLRewriter().on("*", {
			text(text) {
				if (text.text) {
					let modifiedText = text.text;
					for (const [key, value] of Object.entries(variables)) {
						const regex = new RegExp(`{{${key}}}`, "g");
						modifiedText = modifiedText.replace(regex, String(value));
					}
					if (modifiedText !== text.text) {
						text.replace(modifiedText);
					}
				}
			},
		});

		const response = new Response(templateContent, {
			headers: { "Content-Type": "text/html" },
		});

		const transformedResponse = rewriter.transform(response);
		return await transformedResponse.text();
	}

	async sendEmail(options: SendMailOptions): Promise<EmailResult> {
		try {
			let htmlContent: string;
			let textContent: string | undefined;

			if (options.template && options.templateVariables) {
				htmlContent = await this.renderTemplate(
					options.template,
					options.templateVariables,
				);

				if (!options.text) {
					textContent = this.htmlToText(htmlContent);
				} else {
					textContent = options.text;
				}
			} else if (options.html) {
				htmlContent = options.html;
				textContent = options.text;
			} else {
				throw new Error("Either template or html content must be provided");
			}

			const mailOptions: nodemailer.SendMailOptions = {
				from: options.from || mailerConfig.from,
				to: Array.isArray(options.to) ? options.to.join(", ") : options.to,
				cc: options.cc
					? Array.isArray(options.cc)
						? options.cc.join(", ")
						: options.cc
					: undefined,
				bcc: options.bcc
					? Array.isArray(options.bcc)
						? options.bcc.join(", ")
						: options.bcc
					: undefined,
				subject: options.subject,
				text: textContent,
				html: htmlContent,
				attachments: options.attachments,
				replyTo: options.replyTo,
				priority: options.priority,
				headers: options.headers,
			};

			const info = await this.transporter.sendMail(mailOptions);

			echo.debug({
				message: `Email sent successfully to ${options.to}`,
				messageId: info.messageId,
				subject: options.subject,
				template: options.template,
			});

			return {
				success: true,
				messageId: info.messageId,
				response: info.response,
			};
		} catch (error) {
			echo.error({
				message: "Failed to send email:",
				error,
				to: options.to,
				subject: options.subject,
				template: options.template,
			});

			return {
				success: false,
				error:
					error instanceof Error ? error.message : "Unknown error occurred",
			};
		}
	}

	async sendTemplateEmail(
		to: string | string[],
		subject: string,
		templateName: string,
		variables: TemplateVariables,
		options?: Partial<SendMailOptions>,
	): Promise<EmailResult> {
		return this.sendEmail({
			to,
			subject,
			template: templateName,
			templateVariables: variables,
			...options,
		});
	}

	private htmlToText(html: string): string {
		return html
			.replace(/<style[^>]*>.*?<\/style>/gis, "") // remove style tags
			.replace(/<script[^>]*>.*?<\/script>/gis, "") // remove script tags
			.replace(/<[^>]*>/g, "") // remove HTML tags
			.replace(/&nbsp;/g, " ") // replace non-breaking spaces
			.replace(/&amp;/g, "&") // replace ampersands
			.replace(/&lt;/g, "<") // replace less than
			.replace(/&gt;/g, ">") // replace greater than
			.replace(/&quot;/g, '"') // replace quotes
			.replace(/&#39;/g, "'") // replace apostrophes
			.replace(/\s+/g, " ") // replace multiple whitespace with single space
			.trim();
	}
}

const mailerService = new MailerService();
export { MailerService, mailerService };
