import { redis } from "bun";
import { cassandra } from "#lib/database";

import type { ExtendedRequest, HealthResponse, RouteDef } from "#types/server";

const routeDef: RouteDef = {
	method: "GET",
	accepts: "*/*",
	returns: "application/json",
};

async function checkRedisHealth(): Promise<string> {
	try {
		await redis.exists("__health_check__");
		return "healthy";
	} catch {
		return "unhealthy";
	}
}

async function handler(request: ExtendedRequest): Promise<Response> {
	const cassandraHealth = cassandra.getHealthStatus();
	const redisHealth = await checkRedisHealth();

	const isHealthy = cassandraHealth.connected && redisHealth === "healthy";

	const response: HealthResponse = {
		code: isHealthy ? 200 : 503,
		success: isHealthy,
		message: isHealthy
			? "All services are healthy"
			: "One or more services are unhealthy",
		timestamp: new Date().toISOString(),
		requestTime: `${(performance.now() - request.startPerf).toFixed(2)}ms`,
		services: {
			cassandra: {
				status: cassandraHealth.connected ? "healthy" : "unhealthy",
				hosts: cassandraHealth.hosts,
			},
			redis: {
				status: redisHealth,
			},
		},
	};

	return Response.json(response, {
		status: isHealthy ? 200 : 503,
	});
}

export { handler, routeDef };
