import { echo } from "@atums/echo";
import { sessionManager } from "#lib/auth";
import { cassandra } from "#lib/database";

import type {
	ExtendedRequest,
	RouteDef,
	UserInfoResponse,
	UserResponse,
	UserRow,
} from "#types/server";

const routeDef: RouteDef = {
	method: "GET",
	accepts: "*/*",
	returns: "application/json",
};

async function handler(request: ExtendedRequest): Promise<Response> {
	try {
		const { id: identifier } = request.params;

		const session = await sessionManager.getSession(request);

		let userQuery: string;
		let queryParams: string[];
		let targetUser: UserRow | null = null;

		if (!identifier) {
			if (!session) {
				const response: UserInfoResponse = {
					code: 401,
					success: false,
					error: "Not authenticated",
				};
				return Response.json(response, { status: 401 });
			}

			userQuery = `
				SELECT id, username, display_name, email, is_verified, created_at, updated_at
				FROM users WHERE id = ? LIMIT 1
			`;
			queryParams = [session.id];
		} else {
			const isLikelyId = identifier.startsWith("user_");

			if (isLikelyId) {
				userQuery = `
					SELECT id, username, display_name, email, is_verified, created_at, updated_at
					FROM users WHERE id = ? LIMIT 1
				`;
				queryParams = [identifier];
			} else {
				userQuery = `
					SELECT id, username, display_name, email, is_verified, created_at, updated_at
					FROM users WHERE username = ? LIMIT 1
				`;
				queryParams = [identifier];
			}
		}

		const userResult = (await cassandra.execute(userQuery, queryParams)) as {
			rows: UserRow[];
		};

		if (!userResult?.rows || !Array.isArray(userResult.rows)) {
			const response: UserInfoResponse = {
				code: 500,
				success: false,
				error: "Database query failed",
			};
			return Response.json(response, { status: 500 });
		}

		if (userResult.rows.length === 0) {
			if (identifier?.startsWith("user_")) {
				const usernameQuery = `
					SELECT id, username, display_name, email, is_verified, created_at, updated_at
					FROM users WHERE username = ? LIMIT 1
				`;

				const usernameResult = (await cassandra.execute(usernameQuery, [
					identifier,
				])) as {
					rows: UserRow[];
				};

				if (usernameResult.rows.length > 0) {
					targetUser = usernameResult.rows[0] || null;
				}
			}

			if (!targetUser) {
				const response: UserInfoResponse = {
					code: 404,
					success: false,
					error: "User not found",
				};
				return Response.json(response, { status: 404 });
			}
		} else {
			targetUser = userResult.rows[0] || null;
		}

		if (!targetUser) {
			const response: UserInfoResponse = {
				code: 404,
				success: false,
				error: "User not found",
			};
			return Response.json(response, { status: 404 });
		}

		const isOwnProfile = session?.id === targetUser.id;

		let responseUser: UserResponse;

		if (isOwnProfile) {
			responseUser = {
				id: targetUser.id,
				username: targetUser.username,
				displayName: targetUser.display_name,
				email: targetUser.email,
				isVerified: targetUser.is_verified,
				createdAt: targetUser.created_at.toISOString(),
			};
		} else {
			responseUser = {
				id: targetUser.id,
				username: targetUser.username,
				displayName: targetUser.display_name,
				email: "",
				isVerified: targetUser.is_verified,
				createdAt: targetUser.created_at.toISOString(),
			};
		}

		const response: UserInfoResponse = {
			code: 200,
			success: true,
			message: isOwnProfile
				? "User information retrieved successfully"
				: "Public user information retrieved successfully",
			user: responseUser,
		};

		return Response.json(response, { status: 200 });
	} catch (error) {
		echo.error({
			message: "Error retrieving user information",
			error,
		});

		const response: UserInfoResponse = {
			code: 500,
			success: false,
			error: "Internal server error",
		};
		return Response.json(response, { status: 500 });
	}
}

export { handler, routeDef };
