import { echo } from "@atums/echo";
import { redis } from "bun";
import { environment } from "#environment/config";
import { extraValues } from "#environment/extra";
import { cassandra } from "#lib/database";
import { mailerService } from "#lib/mailer";
import { isValidEmail } from "#lib/validation";

import type {
	BaseResponse,
	ExtendedRequest,
	ForgotPasswordRequest,
	RouteDef,
} from "#types/server";

const routeDef: RouteDef = {
	method: "POST",
	accepts: "application/json",
	returns: "application/json",
	needsBody: "json",
};

async function handler(
	_request: ExtendedRequest,
	requestBody: unknown,
): Promise<Response> {
	try {
		const { email } = requestBody as ForgotPasswordRequest;

		if (!email) {
			const response: BaseResponse = {
				code: 400,
				success: false,
				error: "Email is required",
			};
			return Response.json(response, { status: 400 });
		}

		const emailValidation = isValidEmail(email);
		if (!emailValidation.valid) {
			const response: BaseResponse = {
				code: 400,
				success: false,
				error: emailValidation.error || "Invalid email",
			};
			return Response.json(response, { status: 400 });
		}

		const userQuery = `
			SELECT id, username, display_name, email, is_verified
			FROM users WHERE email = ? LIMIT 1
		`;
		const userResult = (await cassandra.execute(userQuery, [
			email.trim().toLowerCase(),
		])) as {
			rows: Array<{
				id: string;
				username: string;
				display_name: string | null;
				email: string;
				is_verified: boolean;
			}>;
		};

		if (!userResult?.rows || userResult.rows.length === 0) {
			const response: BaseResponse = {
				code: 200,
				success: true,
				message: "If the email exists, a password reset link has been sent",
			};
			return Response.json(response, { status: 200 });
		}

		const user = userResult.rows[0];
		if (!user) {
			const response: BaseResponse = {
				code: 200,
				success: true,
				message: "If the email exists, a password reset link has been sent",
			};
			return Response.json(response, { status: 200 });
		}

		const resetToken = Bun.randomUUIDv7();
		const resetKey = `password-reset:${resetToken}`;

		try {
			await redis.set(
				resetKey,
				JSON.stringify({
					userId: user.id,
					email: user.email,
				}),
				"EX",
				1 * 60 * 60, // 1 hour
			);

			const emailVariables = {
				subject: `Password Reset - ${extraValues.companyName}`,
				companyName: extraValues.companyName,
				id: user.id,
				displayName: user.display_name || user.username,
				willExpire: "This link will expire in 1 hour",
				resetUrl: `${environment.frontendFqdn}/user/reset-password?token=${resetToken}`,
				supportEmail: extraValues.supportEmail,
				currentYear: new Date().getFullYear(),
			};

			await mailerService.sendTemplateEmail(
				user.email,
				`Password Reset - ${extraValues.companyName}`,
				"forgot-password",
				emailVariables,
			);

			const response: BaseResponse = {
				code: 200,
				success: true,
				message: "If the email exists, a password reset link has been sent",
			};
			return Response.json(response, { status: 200 });
		} catch (error) {
			await redis.del(resetKey).catch(() => {});

			echo.error({
				message: "Failed to send password reset email",
				error,
				userId: user.id,
				email: user.email,
			});

			const response: BaseResponse = {
				code: 500,
				success: false,
				error: "Failed to send password reset email. Please try again.",
			};
			return Response.json(response, { status: 500 });
		}
	} catch (error) {
		echo.error({
			message: "Password reset request failed",
			error,
		});

		const response: BaseResponse = {
			code: 500,
			success: false,
			error: "Internal server error",
		};
		return Response.json(response, { status: 500 });
	}
}

export { handler, routeDef };
