import { echo } from "@atums/echo";
import { redis } from "bun";
import { sessionManager } from "#lib/auth";
import { cassandra } from "#lib/database";
import { isValidPassword } from "#lib/validation";
import type {
	ExtendedRequest,
	ResetData,
	ResetPasswordRequest,
	ResetPasswordResponse,
	RouteDef,
	UserRow,
} from "#types/server";

const routeDef: RouteDef = {
	method: "POST",
	accepts: "application/json",
	returns: "application/json",
	needsBody: "json",
};

async function handler(
	_request: ExtendedRequest,
	requestBody: unknown,
): Promise<Response> {
	try {
		const {
			token,
			newPassword,
			logoutAllSessions = true,
		} = requestBody as ResetPasswordRequest;

		if (!token || !newPassword) {
			const response: ResetPasswordResponse = {
				code: 400,
				success: false,
				error: "Token and new password are required",
			};
			return Response.json(response, { status: 400 });
		}

		const passwordValidation = isValidPassword(newPassword);
		if (!passwordValidation.valid) {
			const response: ResetPasswordResponse = {
				code: 400,
				success: false,
				error: passwordValidation.error || "Invalid password",
			};
			return Response.json(response, { status: 400 });
		}

		const resetKey = `password-reset:${token}`;
		const resetDataRaw = await redis.get(resetKey);

		if (!resetDataRaw) {
			const response: ResetPasswordResponse = {
				code: 400,
				success: false,
				error: "Invalid or expired reset token",
			};
			return Response.json(response, { status: 400 });
		}

		let resetData: ResetData;
		try {
			resetData = JSON.parse(resetDataRaw);
		} catch {
			await redis.del(resetKey);
			const response: ResetPasswordResponse = {
				code: 400,
				success: false,
				error: "Invalid reset token format",
			};
			return Response.json(response, { status: 400 });
		}

		if (!resetData.userId || !resetData.email) {
			await redis.del(resetKey);
			const response: ResetPasswordResponse = {
				code: 400,
				success: false,
				error: "Invalid reset data",
			};
			return Response.json(response, { status: 400 });
		}

		const userQuery = `
			SELECT id, username, display_name, email, password, is_verified, created_at, updated_at
			FROM users WHERE id = ? LIMIT 1
		`;
		const userResult = (await cassandra.execute(userQuery, [
			resetData.userId,
		])) as { rows: UserRow[] };

		if (!userResult?.rows || userResult.rows.length === 0) {
			await redis.del(resetKey);
			const response: ResetPasswordResponse = {
				code: 404,
				success: false,
				error: "User not found",
			};
			return Response.json(response, { status: 404 });
		}

		const user = userResult.rows[0];
		if (!user) {
			await redis.del(resetKey);
			const response: ResetPasswordResponse = {
				code: 404,
				success: false,
				error: "User not found",
			};
			return Response.json(response, { status: 404 });
		}

		if (user.email !== resetData.email) {
			await redis.del(resetKey);
			const response: ResetPasswordResponse = {
				code: 400,
				success: false,
				error: "Reset token does not match current email address",
			};
			return Response.json(response, { status: 400 });
		}

		const isSamePassword = await Bun.password.verify(
			newPassword,
			user.password,
		);
		if (isSamePassword) {
			const response: ResetPasswordResponse = {
				code: 400,
				success: false,
				error: "New password must be different from current password",
			};
			return Response.json(response, { status: 400 });
		}

		const hashedNewPassword = await Bun.password.hash(newPassword, {
			algorithm: "argon2id",
			memoryCost: 4096,
			timeCost: 3,
		});

		const updateQuery = `
			UPDATE users
			SET password = ?, updated_at = ?
			WHERE id = ?
		`;
		await cassandra.execute(updateQuery, [
			hashedNewPassword,
			new Date(),
			user.id,
		]);

		await redis.del(resetKey);

		let invalidatedCount = 0;
		if (logoutAllSessions) {
			invalidatedCount = await sessionManager.invalidateAllSessionsForUser(
				user.id,
			);
		}

		const response: ResetPasswordResponse = {
			code: 200,
			success: true,
			message: logoutAllSessions
				? `Password reset successfully. Logged out from ${invalidatedCount} session(s).`
				: "Password reset successfully.",
			loggedOutSessions: invalidatedCount,
		};

		return Response.json(response, { status: 200 });
	} catch (error) {
		echo.error({
			message: "Password reset failed",
			error,
		});

		const response: ResetPasswordResponse = {
			code: 500,
			success: false,
			error: "Internal server error",
		};
		return Response.json(response, { status: 500 });
	}
}

export { handler, routeDef };
