import { echo } from "@atums/echo";
import { sessionManager } from "#lib/auth";
import { cassandra } from "#lib/database";
import {
	isValidDisplayName,
	isValidEmail,
	isValidUsername,
} from "#lib/validation";

import type {
	ExtendedRequest,
	RouteDef,
	UpdateInfoRequest,
	UpdateInfoResponse,
	UserResponse,
	UserRow,
} from "#types/server";

const routeDef: RouteDef = {
	method: ["PUT", "PATCH"],
	accepts: "application/json",
	returns: "application/json",
	needsBody: "json",
};

async function handler(
	request: ExtendedRequest,
	requestBody: unknown,
): Promise<Response> {
	try {
		const session = await sessionManager.getSession(request);

		if (!session) {
			const response: UpdateInfoResponse = {
				code: 401,
				success: false,
				error: "Not authenticated",
			};
			return Response.json(response, { status: 401 });
		}

		const { username, displayName, email } = requestBody as UpdateInfoRequest;

		if (
			username === undefined &&
			displayName === undefined &&
			email === undefined
		) {
			const response: UpdateInfoResponse = {
				code: 400,
				success: false,
				error:
					"At least one field must be provided (username, displayName, email)",
			};
			return Response.json(response, { status: 400 });
		}

		const currentUserQuery = `
			SELECT id, username, display_name, email, is_verified, created_at, updated_at
			FROM users WHERE id = ? LIMIT 1
		`;

		const currentUserResult = (await cassandra.execute(currentUserQuery, [
			session.id,
		])) as { rows: UserRow[] };

		if (!currentUserResult?.rows || currentUserResult.rows.length === 0) {
			await sessionManager.invalidateSession(request);

			const response: UpdateInfoResponse = {
				code: 404,
				success: false,
				error: "User not found",
			};
			return Response.json(response, { status: 404 });
		}

		const currentUser = currentUserResult.rows[0];
		if (!currentUser) {
			const response: UpdateInfoResponse = {
				code: 404,
				success: false,
				error: "User not found",
			};
			return Response.json(response, { status: 404 });
		}

		const updates: {
			username?: string;
			displayName?: string | null;
			email?: string;
		} = {};

		if (username !== undefined) {
			const usernameValidation = isValidUsername(username);
			if (!usernameValidation.valid || !usernameValidation.username) {
				const response: UpdateInfoResponse = {
					code: 400,
					success: false,
					error: usernameValidation.error || "Invalid username",
				};
				return Response.json(response, { status: 400 });
			}

			if (usernameValidation.username !== currentUser.username) {
				const existingUsernameQuery =
					"SELECT id FROM users WHERE username = ? LIMIT 1";
				const existingUsernameResult = (await cassandra.execute(
					existingUsernameQuery,
					[usernameValidation.username],
				)) as { rows: Array<{ id: string }> };

				if (
					existingUsernameResult.rows.length > 0 &&
					existingUsernameResult.rows[0]?.id !== session.id
				) {
					const response: UpdateInfoResponse = {
						code: 409,
						success: false,
						error: "Username already exists",
					};
					return Response.json(response, { status: 409 });
				}

				updates.username = usernameValidation.username;
			}
		}

		if (displayName !== undefined) {
			if (displayName === null || displayName.trim() === "") {
				updates.displayName = null;
			} else {
				const displayNameValidation = isValidDisplayName(displayName);
				if (!displayNameValidation.valid) {
					const response: UpdateInfoResponse = {
						code: 400,
						success: false,
						error: displayNameValidation.error || "Invalid display name",
					};
					return Response.json(response, { status: 400 });
				}
				updates.displayName = displayNameValidation.name || null;
			}
		}

		if (email !== undefined) {
			const emailValidation = isValidEmail(email);
			if (!emailValidation.valid) {
				const response: UpdateInfoResponse = {
					code: 400,
					success: false,
					error: emailValidation.error || "Invalid email",
				};
				return Response.json(response, { status: 400 });
			}

			const normalizedEmail = email.trim().toLowerCase();

			if (normalizedEmail !== currentUser.email) {
				const existingEmailQuery =
					"SELECT id FROM users WHERE email = ? LIMIT 1";
				const existingEmailResult = (await cassandra.execute(
					existingEmailQuery,
					[normalizedEmail],
				)) as { rows: Array<{ id: string }> };

				if (
					existingEmailResult.rows.length > 0 &&
					existingEmailResult.rows[0]?.id !== session.id
				) {
					const response: UpdateInfoResponse = {
						code: 409,
						success: false,
						error: "Email already exists",
					};
					return Response.json(response, { status: 409 });
				}

				updates.email = normalizedEmail;
			}
		}

		if (Object.keys(updates).length === 0) {
			const response: UpdateInfoResponse = {
				code: 200,
				success: true,
				message: "No changes required",
				user: {
					id: currentUser.id,
					username: currentUser.username,
					displayName: currentUser.display_name,
					email: currentUser.email,
					isVerified: currentUser.is_verified,
					createdAt: currentUser.created_at.toISOString(),
				},
			};
			return Response.json(response, { status: 200 });
		}

		const updateFields: string[] = [];
		const updateValues: unknown[] = [];

		if (updates.username !== undefined) {
			updateFields.push("username = ?");
			updateValues.push(updates.username);
		}

		if (updates.displayName !== undefined) {
			updateFields.push("display_name = ?");
			updateValues.push(updates.displayName);
		}

		if (updates.email !== undefined) {
			updateFields.push("email = ?");
			updateValues.push(updates.email);
			updateFields.push("is_verified = ?");
			updateValues.push(false);
		}

		updateFields.push("updated_at = ?");
		updateValues.push(new Date());

		updateValues.push(session.id);

		const updateQuery = `
			UPDATE users
			SET ${updateFields.join(", ")}
			WHERE id = ?
		`;

		await cassandra.execute(updateQuery, updateValues);

		const updatedUserResult = (await cassandra.execute(currentUserQuery, [
			session.id,
		])) as { rows: UserRow[] };

		const updatedUser = updatedUserResult.rows[0];
		if (!updatedUser) {
			const response: UpdateInfoResponse = {
				code: 500,
				success: false,
				error: "Failed to fetch updated user data",
			};
			return Response.json(response, { status: 500 });
		}

		if (Object.keys(updates).length > 0) {
			const userAgent = request.headers.get("User-Agent") || "Unknown";
			const updatedSessionPayload = {
				id: updatedUser.id,
				username: updatedUser.username,
				email: updatedUser.email,
				isVerified: updatedUser.is_verified,
				displayName: updatedUser.display_name,
				createdAt: updatedUser.created_at.toISOString(),
				updatedAt: updatedUser.updated_at.toISOString(),
			};

			const sessionCookie = await sessionManager.updateSession(
				request,
				updatedSessionPayload,
				userAgent,
			);

			const responseUser: UserResponse = {
				id: updatedUser.id,
				username: updatedUser.username,
				displayName: updatedUser.display_name,
				email: updatedUser.email,
				isVerified: updatedUser.is_verified,
				createdAt: updatedUser.created_at.toISOString(),
			};

			const response: UpdateInfoResponse = {
				code: 200,
				success: true,
				message: "User information updated successfully",
				user: responseUser,
			};

			return Response.json(response, {
				status: 200,
				headers: {
					"Set-Cookie": sessionCookie,
				},
			});
		}

		const responseUser: UserResponse = {
			id: updatedUser.id,
			username: updatedUser.username,
			displayName: updatedUser.display_name,
			email: updatedUser.email,
			isVerified: updatedUser.is_verified,
			createdAt: updatedUser.created_at.toISOString(),
		};

		const response: UpdateInfoResponse = {
			code: 200,
			success: true,
			message: "User information updated successfully",
			user: responseUser,
		};

		return Response.json(response, { status: 200 });
	} catch (error) {
		echo.error({
			message: "Error updating user information",
			error,
		});

		const response: UpdateInfoResponse = {
			code: 500,
			success: false,
			error: "Internal server error",
		};
		return Response.json(response, { status: 500 });
	}
}

export { handler, routeDef };
