import { echo } from "@atums/echo";
import { redis } from "bun";
import { sessionManager } from "#lib/auth";
import { cassandra } from "#lib/database";

import type {
	ExtendedRequest,
	RouteDef,
	UserResponse,
	VerificationData,
	VerifyEmailResponse,
} from "#types/server";

const routeDef: RouteDef = {
	method: "GET",
	accepts: "*/*",
	returns: "application/json",
};

async function handler(request: ExtendedRequest): Promise<Response> {
	try {
		const { token } = request.query;

		if (!token || typeof token !== "string" || token.trim() === "") {
			const response: VerifyEmailResponse = {
				code: 400,
				success: false,
				error: "Verification token is required",
			};
			return Response.json(response, { status: 400 });
		}

		const verificationKey = `mail-verification:${token}`;
		const verificationDataRaw = await redis.get(verificationKey);

		if (!verificationDataRaw) {
			const response: VerifyEmailResponse = {
				code: 400,
				success: false,
				error: "Invalid or expired verification token",
			};
			return Response.json(response, { status: 400 });
		}

		let verificationData: VerificationData;
		try {
			verificationData = JSON.parse(verificationDataRaw);
		} catch {
			await redis.del(verificationKey);

			const response: VerifyEmailResponse = {
				code: 400,
				success: false,
				error: "Invalid verification token format",
			};
			return Response.json(response, { status: 400 });
		}

		if (!verificationData.userId || !verificationData.email) {
			await redis.del(verificationKey);

			const response: VerifyEmailResponse = {
				code: 400,
				success: false,
				error: "Invalid verification data",
			};
			return Response.json(response, { status: 400 });
		}

		const userQuery = `
			SELECT id, username, display_name, email, is_verified, created_at, updated_at
			FROM users WHERE id = ? LIMIT 1
		`;

		const userResult = (await cassandra.execute(userQuery, [
			verificationData.userId,
		])) as {
			rows: Array<{
				id: string;
				username: string;
				display_name: string | null;
				email: string;
				is_verified: boolean;
				created_at: Date;
				updated_at: Date;
			}>;
		};

		if (!userResult?.rows || userResult.rows.length === 0) {
			await redis.del(verificationKey);

			const response: VerifyEmailResponse = {
				code: 404,
				success: false,
				error: "User not found",
			};
			return Response.json(response, { status: 404 });
		}

		const user = userResult.rows[0];
		if (!user) {
			await redis.del(verificationKey);

			const response: VerifyEmailResponse = {
				code: 404,
				success: false,
				error: "User not found",
			};
			return Response.json(response, { status: 404 });
		}

		if (user.email !== verificationData.email) {
			await redis.del(verificationKey);

			const response: VerifyEmailResponse = {
				code: 400,
				success: false,
				error: "Verification token does not match current email address",
			};
			return Response.json(response, { status: 400 });
		}

		if (user.is_verified) {
			await redis.del(verificationKey);

			const responseUser: UserResponse = {
				id: user.id,
				username: user.username,
				displayName: user.display_name,
				email: user.email,
				isVerified: user.is_verified,
				createdAt: user.created_at.toISOString(),
			};

			const response: VerifyEmailResponse = {
				code: 200,
				success: true,
				message: "Email is already verified",
				user: responseUser,
			};
			return Response.json(response, { status: 200 });
		}

		const updateQuery = `
			UPDATE users
			SET is_verified = ?, updated_at = ?
			WHERE id = ?
		`;

		await cassandra.execute(updateQuery, [true, new Date(), user.id]);

		await redis.del(verificationKey);

		const updatedUserResult = (await cassandra.execute(userQuery, [
			user.id,
		])) as {
			rows: Array<{
				id: string;
				username: string;
				display_name: string | null;
				email: string;
				is_verified: boolean;
				created_at: Date;
				updated_at: Date;
			}>;
		};

		const updatedUser = updatedUserResult.rows[0];
		if (!updatedUser) {
			const response: VerifyEmailResponse = {
				code: 500,
				success: false,
				error: "Failed to fetch updated user data",
			};
			return Response.json(response, { status: 500 });
		}

		const session = await sessionManager.getSession(request);
		let sessionCookie: string | undefined;

		if (session && session.id === user.id) {
			try {
				const userAgent = request.headers.get("User-Agent") || "Unknown";
				const updatedSessionPayload = {
					id: updatedUser.id,
					username: updatedUser.username,
					email: updatedUser.email,
					isVerified: updatedUser.is_verified,
					displayName: updatedUser.display_name,
					createdAt: updatedUser.created_at.toISOString(),
					updatedAt: updatedUser.updated_at.toISOString(),
				};

				sessionCookie = await sessionManager.updateSession(
					request,
					updatedSessionPayload,
					userAgent,
				);
			} catch (sessionError) {
				echo.warn({
					message: "Failed to update session after email verification",
					error: sessionError,
					userId: user.id,
				});
			}
		}

		const responseUser: UserResponse = {
			id: updatedUser.id,
			username: updatedUser.username,
			displayName: updatedUser.display_name,
			email: updatedUser.email,
			isVerified: updatedUser.is_verified,
			createdAt: updatedUser.created_at.toISOString(),
		};

		const response: VerifyEmailResponse = {
			code: 200,
			success: true,
			message: "Email verified successfully",
			user: responseUser,
		};

		return Response.json(response, {
			status: 200,
			headers: {
				"Set-Cookie": sessionCookie ? sessionCookie : "",
			},
		});
	} catch (error) {
		echo.error({
			message: "Error during email verification",
			error,
			token: request.query.token,
		});

		const response: VerifyEmailResponse = {
			code: 500,
			success: false,
			error: "Internal server error",
		};
		return Response.json(response, { status: 500 });
	}
}

export { handler, routeDef };
